@@ -13,12 +13,13 @@ from labelme import utils
 
 def main():
     logger.warning(
-        "This script is aimed to demonstrate how to convert the "
-        "JSON file to a single image dataset."
+        "DEPRECATED: This script will be removed in the near future. "
+        "Please use `labelme_export_json` instead."
     )
     logger.warning(
-        "It won't handle multiple JSON files to generate a "
-        "real-use dataset."
+        "NOTE: This script is aimed to demonstrate how to convert a JSON file "
+        "to a single image dataset. so it won't handle multiple JSON files to "
+        "generate a real-use dataset."
     )
 
     parser = argparse.ArgumentParser()
@@ -63,7 +64,7 @@ def main():
         label_names[value] = name
 
     lbl_viz = imgviz.label2rgb(
-        label=lbl, img=imgviz.asgray(img), label_names=label_names, loc="rb"
+        lbl, imgviz.asgray(img), label_names=label_names, loc="rb"
     )
 
     PIL.Image.fromarray(img).save(osp.join(out_dir, "img.png"))
