@@ -4,6 +4,7 @@ import math
 from qtpy import QtCore
 from qtpy import QtGui
 
+from labelme.logger import logger
 import labelme.utils
 
 
@@ -51,14 +52,21 @@ class Shape(object):
         shape_type=None,
         flags=None,
         group_id=None,
+        description=None,
     ):
         self.label = label
         self.group_id = group_id
         self.points = []
+        self.point_labels = []
+        self.shape_type = shape_type
+        self._shape_raw = None
+        self._points_raw = []
+        self._shape_type_raw = None
         self.fill = False
         self.selected = False
         self.shape_type = shape_type
         self.flags = flags
+        self.description = description
         self.other_data = {}
 
         self._highlightIndex = None
@@ -76,8 +84,18 @@ class Shape(object):
             # is used for drawing the pending line a different color.
             self.line_color = line_color
 
+    def setShapeRefined(self, points, point_labels, shape_type):
+        self._shape_raw = (self.points, self.point_labels, self.shape_type)
+        self.points = points
+        self.point_labels = point_labels
         self.shape_type = shape_type
 
+    def restoreShapeRaw(self):
+        if self._shape_raw is None:
+            return
+        self.points, self.point_labels, self.shape_type = self._shape_raw
+        self._shape_raw = None
+
     @property
     def shape_type(self):
         return self._shape_type
@@ -93,6 +111,7 @@ class Shape(object):
             "line",
             "circle",
             "linestrip",
+            "points",
         ]:
             raise ValueError("Unexpected shape_type: {}".format(value))
         self._shape_type = value
@@ -100,25 +119,53 @@ class Shape(object):
     def close(self):
         self._closed = True
 
-    def addPoint(self, point):
+    def addPoint(self, point, label=1):
         if self.points and point == self.points[0]:
             self.close()
         else:
             self.points.append(point)
+            self.point_labels.append(label)
 
     def canAddPoint(self):
         return self.shape_type in ["polygon", "linestrip"]
 
     def popPoint(self):
         if self.points:
+            if self.point_labels:
+                self.point_labels.pop()
             return self.points.pop()
         return None
 
-    def insertPoint(self, i, point):
+    def insertPoint(self, i, point, label=1):
         self.points.insert(i, point)
+        self.point_labels.insert(i, label)
 
     def removePoint(self, i):
+        if not self.canAddPoint():
+            logger.warning(
+                "Cannot remove point from: shape_type=%r",
+                self.shape_type,
+            )
+            return
+
+        if self.shape_type == "polygon" and len(self.points) <= 3:
+            logger.warning(
+                "Cannot remove point from: shape_type=%r, len(points)=%d",
+                self.shape_type,
+                len(self.points),
+            )
+            return
+
+        if self.shape_type == "linestrip" and len(self.points) <= 2:
+            logger.warning(
+                "Cannot remove point from: shape_type=%r, len(points)=%d",
+                self.shape_type,
+                len(self.points),
+            )
+            return
+
         self.points.pop(i)
+        self.point_labels.pop(i)
 
     def isClosed(self):
         return self._closed
@@ -143,6 +190,7 @@ class Shape(object):
 
             line_path = QtGui.QPainterPath()
             vrtx_path = QtGui.QPainterPath()
+            negative_vrtx_path = QtGui.QPainterPath()
 
             if self.shape_type == "rectangle":
                 assert len(self.points) in [1, 2]
@@ -163,6 +211,15 @@ class Shape(object):
                 for i, p in enumerate(self.points):
                     line_path.lineTo(p)
                     self.drawVertex(vrtx_path, i)
+            elif self.shape_type == "points":
+                assert len(self.points) == len(self.point_labels)
+                for i, (p, l) in enumerate(
+                    zip(self.points, self.point_labels)
+                ):
+                    if l == 1:
+                        self.drawVertex(vrtx_path, i)
+                    else:
+                        self.drawVertex(negative_vrtx_path, i)
             else:
                 line_path.moveTo(self.points[0])
                 # Uncommenting the following line will draw 2 paths
@@ -187,6 +244,11 @@ class Shape(object):
                 )
                 painter.fillPath(line_path, color)
 
+            pen.setColor(QtGui.QColor(255, 0, 0, 255))
+            painter.setPen(pen)
+            painter.drawPath(negative_vrtx_path)
+            painter.fillPath(negative_vrtx_path, QtGui.QColor(255, 0, 0, 255))
+
     def drawVertex(self, path, i):
         d = self.point_size / self.scale
         shape = self.point_type
